/* XmGoBang: (c) 1995-2008 by Andreas S. Krebs */

#include "xmgobang.h"


/* global variables */

  
  
/* local prototypes */

static Widget createPushButton(Widget, String, XtCallbackProc);
static Widget createSeparator(Widget, String);
static Widget createLabel(Widget, String);

static void setMenuItemsSensitive(Boolean);
static void newGameCB(Widget, XtPointer, XtPointer); 
static void newQuestionCB(Widget, XtPointer, XtPointer);
static void timerCB(XtPointer, XtIntervalId *);


/* static variables */

static Widget graphicWidget, player1TypeW, player2TypeW;
static Widget playModeW, cheatModeW;



/************************************************************************
 *									*
 * create the widgets for the pulldown menu				*
 *									*
 ************************************************************************/

Widget initMenuWidget(Widget parentW)
{
  Widget   menuBarW, menuW, submenuW, buttonW, radioW;
  Arg	   args[10];
  Cardinal n;

  menuBarW=XmCreateMenuBar(parentW, "menuBar", (Arg *)NULL, 0);
  XtManageChild(menuBarW);
  
  menuW=XmCreatePulldownMenu(menuBarW, "gameMenu", NULL, 0);
  createPushButton(menuW, "new", (XtCallbackProc)newQuestionCB);  
  createSeparator(menuW, "menuSeparator");
  createPushButton(menuW, "exit", (XtCallbackProc)exitQuestionCB); 
  
  n=0;
  XtSetArg(args[n], XmNsubMenuId, menuW); n++;
  XtCreateManagedWidget("game", xmCascadeButtonWidgetClass, menuBarW, args, n);

  
  menuW=XmCreatePulldownMenu(menuBarW, "optionsMenu", NULL, 0);


  n=0;
  XtSetArg(args[n], XmNentryClass, xmToggleButtonWidgetClass);	n++;
  XtSetArg(args[n], XmNisHomogeneous, True);			n++;
  XtSetArg(args[n], XmNradioBehavior, XmONE_OF_MANY);		n++;
  submenuW=XmCreatePulldownMenu(menuW, "startPlayerMenu", args, n);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++; 
  if(XGoGame.StartPlayer==0)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("player1", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)startPlayerCB,
		(XtPointer)0);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.StartPlayer==2)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("player2", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)startPlayerCB,
		(XtPointer)1);
  

  n=0;
  XtSetArg(args[n], XmNsubMenuId, submenuW); n++;
  XtCreateManagedWidget("startPlayer", xmCascadeButtonWidgetClass,
			menuW, args, n);

  
  createSeparator(menuW, "menuSeparator");
  
  n=0;
  XtSetArg(args[n], XmNentryClass, xmToggleButtonWidgetClass);	n++;
  XtSetArg(args[n], XmNisHomogeneous, True);			n++;
  XtSetArg(args[n], XmNradioBehavior, XmONE_OF_MANY);		n++;
  submenuW=XmCreatePulldownMenu(menuW, "player1Menu", args, n);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++; 
  if(XGoGame.Player[0].Type==PLAYER_HUMAN)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("playerHuman", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playerTypeCB,
		(XtPointer)PLAYER1_HUMAN);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.Player[0].Type==PLAYER_COMPUTER)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("playerComputer", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playerTypeCB,
		(XtPointer)PLAYER1_COMPUTER);
  

  n=0;
  XtSetArg(args[n], XmNsubMenuId, submenuW); n++;
  player1TypeW=XtCreateManagedWidget("player1", xmCascadeButtonWidgetClass,
				     menuW, args, n);

  
  n=0;
  XtSetArg(args[n], XmNentryClass, xmToggleButtonWidgetClass);	n++;
  XtSetArg(args[n], XmNisHomogeneous, True);			n++;
  XtSetArg(args[n], XmNradioBehavior, XmONE_OF_MANY);		n++;
  submenuW=XmCreatePulldownMenu(menuW, "player2Menu", args, n);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.Player[1].Type==PLAYER_HUMAN)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("playerHuman", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playerTypeCB,
		(XtPointer)PLAYER2_HUMAN);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.Player[1].Type==PLAYER_COMPUTER)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("playerComputer", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playerTypeCB,
		(XtPointer)PLAYER2_COMPUTER);
  

  n=0;
  XtSetArg(args[n], XmNsubMenuId, submenuW); n++;
  player2TypeW=XtCreateManagedWidget("player2", xmCascadeButtonWidgetClass,
				     menuW, args, n);
  
  createSeparator(menuW, "menuSeparator");
  
  n=0;
  XtSetArg(args[n], XmNentryClass, xmToggleButtonWidgetClass);	n++;
  XtSetArg(args[n], XmNisHomogeneous, True);			n++;
  XtSetArg(args[n], XmNradioBehavior, XmONE_OF_MANY);		n++;
  submenuW=XmCreatePulldownMenu(menuW, "playModeMenu", args, n);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.PlayMode==PLAY_NORMAL)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("normal", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playModeCB,
		(XtPointer)PLAY_NORMAL);
  
  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.PlayMode==PLAY_OFFENSIVE)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("offensive", xmToggleButtonWidgetClass, 
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playModeCB,
		(XtPointer)PLAY_OFFENSIVE);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.PlayMode==PLAY_DEFENSIVE)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("defensive", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)playModeCB,
		(XtPointer)PLAY_DEFENSIVE);
  
  
  n=0;
  XtSetArg(args[n], XmNsubMenuId, submenuW); n++;
  playModeW=XtCreateManagedWidget("playMode", xmCascadeButtonWidgetClass, 
				  menuW, args, n);


  n=0;
  XtSetArg(args[n], XmNentryClass, xmToggleButtonWidgetClass);	n++;
  XtSetArg(args[n], XmNisHomogeneous, True);			n++;
  XtSetArg(args[n], XmNradioBehavior, XmONE_OF_MANY);		n++;
  submenuW=XmCreatePulldownMenu(menuW, "cheatModeMenu", args, n);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.CheatMode==CHEAT_NONE)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("cheatnone", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)cheatModeCB,
		(XtPointer)CHEAT_NONE);
  
  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.CheatMode==CHEAT_HINT)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("cheathint", xmToggleButtonWidgetClass, 
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)cheatModeCB,
		(XtPointer)CHEAT_HINT);

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;  
  if(XGoGame.CheatMode==CHEAT_UNDO)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("cheatundo", xmToggleButtonWidgetClass,
				submenuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)cheatModeCB,
		(XtPointer)CHEAT_UNDO);
  
  
  n=0;
  XtSetArg(args[n], XmNsubMenuId, submenuW); n++;
  cheatModeW=XtCreateManagedWidget("cheatMode", xmCascadeButtonWidgetClass, 
				  menuW, args, n);

#ifdef SOUND_SUPPORT  
  createSeparator(menuW, "menuSeparator");  

  n=0;
  XtSetArg(args[n], XmNvisibleWhenOff, True);	n++;
  if(XGoGame.SoundOn==True)
    { XtSetArg(args[n], XmNset, True);		n++; }
  buttonW=XtCreateManagedWidget("sounds", xmToggleButtonWidgetClass,
				menuW, args, n);
  XtAddCallback(buttonW, XmNvalueChangedCallback, (XtCallbackProc)soundCB,
		NULL);
#endif  
  
  n=0;
  XtSetArg(args[n], XmNsubMenuId, menuW); n++;
  XtCreateManagedWidget("options", xmCascadeButtonWidgetClass, menuBarW, args, n);

  
  menuW=XmCreatePulldownMenu(menuBarW, "helpMenu", NULL, 0);
  createPushButton(menuW, "about", (XtCallbackProc)aboutCB);  
  createPushButton(menuW, "onXmGoBang", (XtCallbackProc)helpCB); 
  
  n=0;
  XtSetArg(args[n], XmNsubMenuId, menuW); n++;
  buttonW=XtCreateManagedWidget("help", xmCascadeButtonWidgetClass, menuBarW, args, n);

  n=0;
  XtSetArg(args[n], XmNmenuHelpWidget, buttonW); n++;
  XtSetValues(menuBarW, args, n);
  
  return menuBarW;
}


/************************************************************************
 *									*
 * create a managed push button widget and add activate callback 	*
 * procedure								*
 *									*
 ************************************************************************/

static Widget createPushButton(Widget parentW, String name, 
			       XtCallbackProc callback)
{
  Widget buttonW;
  
  buttonW=XtCreateManagedWidget(name, xmPushButtonWidgetClass, parentW,
				(Arg *)NULL, 0);
  XtAddCallback(buttonW, XmNactivateCallback, callback, (XtPointer)NULL);
  
  return buttonW;
}


/************************************************************************
 *									*
 * create a separator widget 						*
 *									*
 ************************************************************************/

static Widget createSeparator(Widget parentW, String name)
{
  Widget separatorW;
  
  separatorW=XtCreateManagedWidget(name, xmSeparatorWidgetClass,
				   parentW, (Arg *)NULL, 0);
  
  return separatorW;
}


/************************************************************************
 *									*
 * create a label widget 						*
 *									*
 ************************************************************************/

static Widget createLabel(Widget parentW, String name)
{
  Widget labelW;
  
  labelW=XtCreateManagedWidget(name, xmLabelWidgetClass,
			       parentW, (Arg *)NULL, 0);
  
  return labelW;
}


/************************************************************************
 *									*
 * Callback procedure for the `exit`-menu item:  			*
 *   this procedure opens a message box and asks for a confirmation	*
 *   to exit the program						*
 *									*
 ************************************************************************/

void exitQuestionCB(Widget widget, XtPointer client_data, 
	 	    XtPointer call_data)
{
  static Widget questionW=NULL;
  static Widget dialogShell=NULL;

  if(questionW==NULL)
  {
    Arg         args[10];
    Cardinal    n;
    
    n=0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
    questionW=XmCreateWarningDialog(applShell, "exitQuestion", args, n);

    dialogShell=XtParent(questionW);

    XtAddCallback(questionW, XmNokCallback, exitCB, NULL);
    XtAddCallback(questionW, XmNcancelCallback, popdownCB, (XtPointer)dialogShell);

    XtUnmanageChild(XmMessageBoxGetChild(questionW,XmDIALOG_HELP_BUTTON));

    setCloseCallback(dialogShell, (XtCallbackProc)popdownCB, (XtPointer)dialogShell);    

    XtManageChild(questionW);
  }
  else
    XtPopup(dialogShell, XtGrabExclusive);
}


/************************************************************************
 *									*
 * Callback procedure for the `new`-menu item:  			*
 *   this procedure opens a message box and asks for a confirmation	*
 *   to start a new game						*
 *									*
 ************************************************************************/

static void newQuestionCB(Widget widget , XtPointer client_data, XtPointer call_data)
{
  static Widget questionW=NULL;
  static Widget dialogShell=NULL;

  if(XGoGame.GameStatus==GAME_FINISHED || XGoGame.GameStatus==GAME_NEW)
  {
    newGame();
    return;
  }
  
  if(questionW==NULL)
  {
    Arg       	args[10];
    Cardinal    n;
    
    n=0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
    questionW=XmCreateQuestionDialog(applShell, "newQuestion", args, n);

    dialogShell=XtParent(questionW);
    
    XtAddCallback(questionW, XmNokCallback, popdownCB, (XtPointer)dialogShell);
    XtAddCallback(questionW, XmNokCallback, newGameCB, NULL);
    XtAddCallback(questionW, XmNcancelCallback, popdownCB, (XtPointer)dialogShell);

    XtUnmanageChild(XmMessageBoxGetChild(questionW,XmDIALOG_HELP_BUTTON));

    setCloseCallback(dialogShell, (XtCallbackProc)popdownCB, (XtPointer)dialogShell);
    
    XtManageChild(questionW);
  }
  else
    XtPopup(dialogShell, XtGrabExclusive);
}


/************************************************************************
 *									*
 * Callback procedure to close a message box				*
 *									*
 ************************************************************************/

void popdownCB(Widget widget, XtPointer client_data,  XtPointer call_data)
{
  XtPopdown((Widget)client_data);
}


/************************************************************************
 *									*
 * Callback procedure to start a new game				*
 *									*
 ************************************************************************/

static void newGameCB(Widget widget, XtPointer client_data, 
		      XtPointer call_data)
{
  if(XGoGame.GameStatus==GAME_RUNNING || XGoGame.GameStatus==GAME_PAUSED)
    playSound(NEW_SOUND);
  
  newGame();
}


/************************************************************************
 *									*
 * create form widget, child buttons and a drawing area 		*
 *									*
 ************************************************************************/

static Widget movePlayerW, timeW[2], undoWidget, continueWidget, hintWidget;
static Widget redoWidget, formW;

Widget initFormWidget(Widget parentW)
{
  Pixel    bg;
  XmString cstring;
  String   txt;
  Widget   formW2, cmdBoxW, timerBoxW, toMoveBoxW;
  Arg	   args[20];
  Cardinal n;
		
  formW=XtCreateManagedWidget("form", xmFormWidgetClass, parentW, 
			      (Arg *)NULL, 0);  
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); 		n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM);		n++;
  XtSetArg(args[n], XmNorientation, XmHORIZONTAL);		n++;
  XtSetArg(args[n], XmNpacking, XmPACK_TIGHT);			n++;	
  XtSetArg(args[n], XmNentryAlignment, XmALIGNMENT_CENTER);	n++;
  cmdBoxW=XtCreateManagedWidget("commandBox", xmRowColumnWidgetClass,
	 		        formW, args, n);

  continueWidget=createPushButton(cmdBoxW, "continue", pauseCB);
  hintWidget=createPushButton(cmdBoxW, "hint", hintCB);
  undoWidget=createPushButton(cmdBoxW, "undo", undoCB);
  redoWidget=createPushButton(cmdBoxW, "redo", redoCB);  

  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET);  	n++;
  XtSetArg(args[n], XmNtopWidget, cmdBoxW);			n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM);		n++;
  XtSetArg(args[n], XmNorientation, XmHORIZONTAL);		n++;
  XtSetArg(args[n], XmNentryAlignment, XmALIGNMENT_CENTER);	n++;
  timerBoxW=XtCreateManagedWidget("timerBox", xmRowColumnWidgetClass,
	 		         formW, args, n);


  n=0;
  XtSetArg(args[n], XmNbackground, &bg);   n++;
  XtGetValues(timerBoxW, args, n);


  txt=XtMalloc(strlen(XGoGame.Player[0].Name)+4);
  strcpy(txt, XGoGame.Player[0].Name);
  strcat(txt, ":");
  cstring=XmStringCreateSimple(txt);
  XtFree(txt);
  
  n=0;
  XtSetArg(args[n], XmNlabelString, cstring);	 		n++;
  if(XGoGame.Player[0].Color != bg)
  {
    XtSetArg(args[n], XmNforeground, XGoGame.Player[0].Color); 	n++;
  }
  XtCreateManagedWidget("playerOne", xmLabelWidgetClass, timerBoxW, args, n);  
  XmStringFree(cstring);
  

  n=0;
  if(XGoGame.Player[0].Color != bg)
  {
    XtSetArg(args[n], XmNforeground, XGoGame.Player[0].Color); 	n++;
  }
  timeW[0]=XtCreateManagedWidget("timeOne", xmLabelWidgetClass, timerBoxW, args, n);
    
  
  createLabel(timerBoxW, "  ");

  txt=XtMalloc(strlen(XGoGame.Player[1].Name)+4);
  strcpy(txt, XGoGame.Player[1].Name);
  strcat(txt, ":");
  cstring=XmStringCreateSimple(txt);
  XtFree(txt);  
  
  n=0;
  XtSetArg(args[n], XmNlabelString, cstring); 			n++;
  if(XGoGame.Player[1].Color != bg)
  {
    XtSetArg(args[n], XmNforeground, XGoGame.Player[1].Color); 	n++;
  }
  XtCreateManagedWidget("playerTwo", xmLabelWidgetClass, timerBoxW, args, n);  
  XmStringFree(cstring);
   
  n=0;
  if(XGoGame.Player[1].Color != bg)
  {
    XtSetArg(args[n], XmNforeground, XGoGame.Player[1].Color); 	n++;
  }
  timeW[1]=XtCreateManagedWidget("timeTwo", xmLabelWidgetClass, timerBoxW, args, n);
 
  
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); 		n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET);	n++;
  XtSetArg(args[n], XmNleftWidget, cmdBoxW);			n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET);	n++;
  XtSetArg(args[n], XmNbottomWidget, timerBoxW);		n++;
  toMoveBoxW=XtCreateManagedWidget("toMoveBox", xmFormWidgetClass,
 			     	    formW, args, n);
    	
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); 		n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM);	n++;
  movePlayerW=XtCreateManagedWidget("mPlayer", xmLabelWidgetClass, toMoveBoxW,
				    args, n);
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); 		n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM);	n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); 	n++;
  XtSetArg(args[n], XmNleftWidget, movePlayerW);		n++;
  XtCreateManagedWidget("toMove", xmLabelWidgetClass, toMoveBoxW, args, n);

  
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); 	n++;
  XtSetArg(args[n], XmNtopWidget, timerBoxW);			n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM);		n++;  
  XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM);		n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM);	n++;
  XtSetArg(args[n], XmNwidth, XGoGame.Width+4);			n++;
  XtSetArg(args[n], XmNheight, XGoGame.Height+4);	 	n++;
  formW2=XtCreateManagedWidget("gfxform", xmFormWidgetClass, formW, 
			       args, n);
  
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM);	 	n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM);		n++;  
  XtSetArg(args[n], XmNborderWidth, 2);				n++;								
  XtSetArg(args[n], XmNwidth, XGoGame.Width);			n++;
  XtSetArg(args[n], XmNheight, XGoGame.Height);	 		n++;
  XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE);		n++;
  XtSetArg(args[n], XmNforeground, XGoGame.BoardColor);		n++;
  XtSetArg(args[n], XmNbackground, XGoGame.BoardBackground);	n++;
  XtSetArg(args[n], XmNborderColor, XGoGame.BoardBorderColor);	n++;
  graphicWidget=XtCreateManagedWidget("graphic", xmDrawingAreaWidgetClass,
 	  		              formW2, args, n);

  XtAddCallback(graphicWidget, XmNexposeCallback, exposeCB, NULL);
  XtAddCallback(graphicWidget, XmNinputCallback, gfxinputCB, NULL);
  
  createCursors(graphicWidget);
  initGCs(graphicWidget);
  
  return formW;
}


/************************************************************************
 *									*
 * set widget sensitive True, to enable widget  			*
 *									*
 ************************************************************************/

void switchOnWidget(short widgetID)
{
  Widget widget;  

  if(XGoGame.CheatMode==CHEAT_NONE || 
     (widgetID!=HINT_WIDGET && XGoGame.CheatMode!=CHEAT_UNDO))
    return;
    
  if(widgetID==UNDO_WIDGET)
    widget=undoWidget;
  else if(widgetID==REDO_WIDGET)
    widget=redoWidget;
  else if(widgetID==HINT_WIDGET)
    widget=hintWidget;
  

  XtSetSensitive(widget, True); 
}


/************************************************************************
 *									*
 * set widget sensitive False, to disable widget 			*
 *									*
 ************************************************************************/

void switchOffWidget(short widgetID)
{
  Widget widget;  
  
  if(widgetID==UNDO_WIDGET)
    widget=undoWidget;
  else if(widgetID==REDO_WIDGET)
    widget=redoWidget;
  else if(widgetID==HINT_WIDGET)
    widget=hintWidget;

  XtSetSensitive(widget, False); 
}


/************************************************************************
 *									*
 * set menu item sensitive, to disable or enable sub menus 		*
 *									*
 ************************************************************************/

static void setMenuItemsSensitive(Boolean sens)
{
  XtSetSensitive(player1TypeW, sens); 
  XtSetSensitive(player2TypeW, sens); 
  XtSetSensitive(playModeW, sens); 
  XtSetSensitive(cheatModeW, sens); 
}


/************************************************************************
 *									*
 * change label of the `pause`-button depending on the current 		*
 * GameStatus								*
 *									*
 ************************************************************************/

static char *statusText[]= 
{ 
  "Start",
  "Pause",
  "Continue",
  "New"
};

void setGameStatus(short status)
{
  XmString	cstring;
  Arg		args[10];
  Cardinal	n;

  XGoGame.GameStatus=status;  
    
  if(status==GAME_RUNNING)
    switchOnWidget(HINT_WIDGET);
  else
    switchOffWidget(HINT_WIDGET);
  
  if(status==GAME_NEW)
    setMenuItemsSensitive(True);
  else
    setMenuItemsSensitive(False);
  
  if(status==GAME_FINISHED || status==GAME_PAUSED)
  {
    setPlayerTimer(0);	/* redisplay both timers */
    setPlayerTimer(1);

    setPlayersTurn(XGoGame.PlayersTurn);	/* redisplay players turn */  
  }
  
  cstring=XmStringCreateSimple(statusText[status]);
  
  n=0;
  XtSetArg(args[n], XmNlabelString, cstring); 	n++;
  XtSetValues(continueWidget, args, n);
  XmStringFree(cstring);
}


/************************************************************************
 *									*
 * set the players turn, by changing the label of the mplayer widget 	*
 *									*
 ************************************************************************/

void setPlayersTurn(short actPlayer)
{
  XmString 	cstring;
  Pixel         bg;
  Arg 	   	args[10];
  Cardinal 	n;
  
  cstring=XmStringCreateSimple(XGoGame.Player[actPlayer].Name);
  
  n=0;
  XtSetArg(args[n], XmNbackground, &bg);    n++;
  XtGetValues(movePlayerW, args, n);

  n=0;
  if(XGoGame.Player[actPlayer].Color != bg)
  {
    XtSetArg(args[n], XmNforeground, XGoGame.Player[actPlayer].Color);	n++;
  }
  XtSetArg(args[n], XmNlabelString, cstring);				n++;
  XtSetValues(movePlayerW, args, n);  
  XmStringFree(cstring);
  
  setCursor(actPlayer);
  XGoGame.PlayersTurn=actPlayer;  
  startPlayerTimer(actPlayer);
}


/************************************************************************
 *									*
 * start game timer for player <playerNr> 				*
 *									*
 ************************************************************************/

static XtIntervalId timerID=0;

void startPlayerTimer(short playerNr)
{
  if(timerID!=0)	/* if there is another timer, remove it */
    XtRemoveTimeOut(timerID);
  
  timerID=XtAppAddTimeOut(context, 1000, timerCB, (XtPointer)(long)playerNr);
}


/************************************************************************
 *									*
 * Callback procedure for timer:  					*
 *   this procedure updates the players clock and starts the timer	*
 *   again if necessary							*
 *									*
 ************************************************************************/

static void timerCB(XtPointer client_data, XtIntervalId *idAdr)
{
  short       playerNr;

  playerNr=(short)(long)client_data;
     
  if(playerNr==XGoGame.PlayersTurn && timerID==*idAdr &&
     XGoGame.GameStatus==GAME_RUNNING) 
  {  
    XGoGame.Player[playerNr].Seconds++;
    setPlayerTimer(playerNr);
      
    timerID=XtAppAddTimeOut(context, 1000, timerCB, client_data);  
  }
  else         
    timerID=0;
}


/************************************************************************
 *									*
 * change label of clock widget, to display new time 			*
 *									*
 ************************************************************************/

void setPlayerTimer(short playerNr)
{
  static char timetxt[10]="00:00:00";
  XmString    cstring;
  short       n;
  long        secs;
  Arg         args[4];

  secs=XGoGame.Player[playerNr].Seconds;
  
  n=secs/3600;	/* number of hours */
  secs%=3600;
  timetxt[0]=n/10+'0';
  timetxt[1]=n%10+'0';
  
  n=secs/60;
  timetxt[3]=n/10+'0';
  timetxt[4]=n%10+'0';
  
  n=secs%60;
  timetxt[6]=n/10+'0';
  timetxt[7]=n%10+'0';
  
  cstring=XmStringCreateSimple(timetxt);
  
  n=0;
  XtSetArg(args[n], XmNlabelString, cstring);	n++;
  XtSetValues(timeW[playerNr], args, n);
  XmStringFree(cstring);  
}
