/* XmGoBang: (c) 1995-2008 by Andreas S. Krebs */

#include "xmgobang.h"

#include "pixmaps/chip.xpm"

/* local prototypes */

static void setcolor(char *, Pixel);
static void showWinMessage(Boolean, short);
static void setWinner(Widget, Boolean, short);
static void getUndoImages();
static void drawShadowRectangle(int, int, int, int, Pixel, Pixel);


/* local variables */

static Pixmap 	gfxPixmap;
static GC 	image_pixmapGC, draw_pixmapGC, win_pixmapGC;
static GC	image_windowGC;
static Widget   gfxWidget;

static XImage 	*playerImage[3];
static XImage	*undoImage[3][3];


/************************************************************************
 *									*
 * create images from xpm data					 	*
 *  									*
 ************************************************************************/

static char     cols_txt[][30]={ "  c #", ". c #", "X c #", "o c #" };

void createXImages()
{
  XImage   *ximage;
  Pixel    bg;

  chip_xpm[1]=cols_txt[0];
  chip_xpm[2]=cols_txt[1];
  chip_xpm[3]=cols_txt[2];
  chip_xpm[4]=cols_txt[3];

  setcolor(&cols_txt[0][5], XGoGame.BoardBackground);
  setcolor(&cols_txt[1][5], XGoGame.Player[0].TopColor);
  setcolor(&cols_txt[2][5], XGoGame.Player[0].Color);
  setcolor(&cols_txt[3][5], XGoGame.Player[0].BottomColor);
  
  XpmCreateImageFromData(display, chip_xpm, &playerImage[0], 
			 &ximage, NULL);
  
  setcolor(&cols_txt[1][5], XGoGame.Player[1].TopColor);
  setcolor(&cols_txt[2][5], XGoGame.Player[1].Color);
  setcolor(&cols_txt[3][5], XGoGame.Player[1].BottomColor);
  
  XpmCreateImageFromData(display, chip_xpm, &playerImage[1], 
			 &ximage, NULL);

  setcolor(&cols_txt[1][5], XGoGame.HintTopColor);
  setcolor(&cols_txt[2][5], XGoGame.HintColor);
  setcolor(&cols_txt[3][5], XGoGame.HintBottomColor);
  
  XpmCreateImageFromData(display, chip_xpm, &playerImage[2], 
			 &ximage, NULL);

}


/************************************************************************
 *									*
 * set rgb-text for according to the color defined by <pix>	 	*
 *  									*
 ************************************************************************/

static void setcolor(char *cs, Pixel pix)
{
  int    r, g, b;
  XColor xcol;
  
  xcol.pixel=pix;
  XQueryColor(display, DefaultColormapOfScreen(XtScreen(gfxWidget)), &xcol);

  r=xcol.red%0x100;
  g=xcol.green%0x100;
  b=xcol.blue%0x100;

  sprintf(cs, "%x%x%x%x%x%x", (r/0x10), (r%0x10), (g/0x10), (g%0x10), (b/0x10), (b%0x10) );
}


/************************************************************************
 *									*
 * create and initialize all needed GCs				 	*
 *  									*
 ************************************************************************/

void initGCs(Widget widget)
{
  XGCValues	values;
  unsigned long mask;
  Window	root;
  Screen	*screen;
  Arg		args[10];
  Cardinal	n;
  
  gfxWidget=widget;
  screen=XtScreen(widget);
  root=RootWindowOfScreen(screen);

  gfxPixmap=XCreatePixmap(display, root, XGoGame.Width, XGoGame.Height,
			  DefaultDepthOfScreen(screen));

  
  values.foreground=XGoGame.BoardColor;
  values.background=XGoGame.BoardBackground;
  mask=GCForeground|GCBackground;
  image_pixmapGC=XCreateGC(display, gfxPixmap, mask, &values);
  image_windowGC=XCreateGC(display, root, mask, &values);

  values.foreground=XGoGame.BoardBackground;
  mask=GCForeground;  
  draw_pixmapGC=XCreateGC(display, gfxPixmap, mask, &values);
  
  values.cap_style=CapRound;
  values.line_width=IMAGE_SIZE-8;
  mask=GCForeground|GCCapStyle|GCLineWidth;
  win_pixmapGC=XCreateGC(display, gfxPixmap, mask, &values);
}


/************************************************************************
 *									*
 * draw a chip for player <playerNo> at the given field <x>, <y>	*
 *  									*
 ************************************************************************/

void drawChipAt(int x, int y, short playerNo)
{
  static short hx=-1, hy=-1;	/* coordinates for hint */
  
  if(hx>=0 && hy>=0)
  {
    undoChipAt(hx, hy);
    
    hx=hy=-1;
  }

  if(playerNo==PLAYER_HINT)
  {
    hx=x;
    hy=y;
  }
  else
    setField(x, y, playerNo+1);
  
  x*=FIELD_SIZE;
  x+=BORDER_SIZE+1;
  
  y*=FIELD_SIZE;
  y+=BORDER_SIZE+1;
  
  XPutImage(display, gfxPixmap, image_pixmapGC,
	    playerImage[playerNo], 0, 0, x, y, IMAGE_SIZE, IMAGE_SIZE);

  XPutImage(display, XtWindow(gfxWidget), image_windowGC,
	    playerImage[playerNo], 0, 0, x, y, IMAGE_SIZE, IMAGE_SIZE);
}


/************************************************************************
 *									*
 * remove a chip from the given field <x>, <y>			 	*
 *  									*
 ************************************************************************/

void undoChipAt(int x, int y)
{
  short i, j;

  setField(x, y, FIELD_EMPTY);
  
  if(x==0) 
    i=0;
  else if(x==(XGoGame.Columns-1))
    i=2;
  else
    i=1;

  if(y==0) 
    j=0;
  else if(y==(XGoGame.Rows-1))
    j=2;
  else
    j=1;
  
  x*=FIELD_SIZE;
  x+=BORDER_SIZE+1;
  
  y*=FIELD_SIZE;
  y+=BORDER_SIZE+1;
  
  XPutImage(display, gfxPixmap, image_pixmapGC,
	    undoImage[i][j], 0, 0, x, y, IMAGE_SIZE, IMAGE_SIZE);

  XPutImage(display, XtWindow(gfxWidget), image_windowGC,
	    undoImage[i][j], 0, 0, x, y, IMAGE_SIZE, IMAGE_SIZE);
}


/************************************************************************
 *									*
 * draw the game board 						 	*
 *  									*
 ************************************************************************/

void drawBoard()
{
  int x, y;
  short i,j;
  
  XSetForeground(display, draw_pixmapGC, XGoGame.BoardBackground);
  XFillRectangle(display, gfxPixmap, draw_pixmapGC, 0, 0,
		 XGoGame.Width, XGoGame.Height);

  XSetForeground(display, draw_pixmapGC, XGoGame.BoardColor);

  XDrawRectangle(display, gfxPixmap, draw_pixmapGC, 
		 FIELD_OFFSET-1, FIELD_OFFSET-1, 
		 (XGoGame.Columns-1)*FIELD_SIZE+2, (XGoGame.Rows-1)*FIELD_SIZE+2);

  drawShadowRectangle(FIELD_OFFSET-2, FIELD_OFFSET-2, 
	   	      XGoGame.Width-FIELD_OFFSET+1, XGoGame.Height-FIELD_OFFSET+1,		      
		      XGoGame.BoardTopColor, XGoGame.BoardBottomColor);

  XSetForeground(display, draw_pixmapGC, XGoGame.BoardColor);
  x=y=FIELD_OFFSET;
  for(i=0;i<XGoGame.Columns;i++)
  {
    XDrawLine(display, gfxPixmap, draw_pixmapGC, 
	      x, FIELD_OFFSET, x, XGoGame.Height-FIELD_OFFSET);
    x+=FIELD_SIZE;
  }

  for(i=0;i<XGoGame.Rows;i++)
  {
    XDrawLine(display, gfxPixmap, draw_pixmapGC, 
	      FIELD_OFFSET, y, XGoGame.Width-FIELD_OFFSET, y);
    y+=FIELD_SIZE;
  }

  x=FIELD_OFFSET+1;
  for(i=0;i<(XGoGame.Columns-1);i++)
  {
    y=FIELD_OFFSET+1;
    for(j=0;j<(XGoGame.Rows-1);j++)
    {
      drawShadowRectangle(x, y, x+FIELD_SIZE-2, y+FIELD_SIZE-2,
			  XGoGame.BoardBottomColor, XGoGame.BoardTopColor);
      y+=FIELD_SIZE;
    }

    x+=FIELD_SIZE;
  }

  getUndoImages();

  XCopyArea(display, gfxPixmap, XtWindow(gfxWidget), image_windowGC,
	    0, 0, XGoGame.Width, XGoGame.Height, 0, 0);
}


/************************************************************************
 *									*
 * get the 9 possible images, to remove a chip from a field		*
 *  									*
 ************************************************************************/

static void getUndoImages()
{
  static Boolean gotImages=False;
  short i,j, x, y;
  unsigned long mask;
  
  if(gotImages==False)
  {  
    mask=(2 << DefaultDepthOfScreen(XtScreen(gfxWidget)))-1;
    for(i=0; i<3; i++)
    {
      x=(i>=2) ? (XGoGame.Columns-1) : i;
    
      x*=FIELD_SIZE;
      x+=BORDER_SIZE+1;
      
      for(j=0; j<3; j++)
      {
        y=(j>=2) ? (XGoGame.Rows-1) : j;

	y*=FIELD_SIZE;
	y+=BORDER_SIZE+1;
        undoImage[i][j]=XGetImage(display, gfxPixmap, x, y,
				  IMAGE_SIZE, IMAGE_SIZE,  mask, ZPixmap);
      }
    }
    gotImages=True;
  }
}


/************************************************************************
 *									*
 * draw a rectangle with a top and bottom shadow	 	 	*
 *  									*
 ************************************************************************/

static void drawShadowRectangle(int x1, int y1, int x2, int y2, 
				Pixel topColor, Pixel bottomColor)
{
  int h;
  
  if(x1 > x2) { h=x1; x1=x2; x2=h; }
  if(y1 > y2) { h=y1; y1=y2; y2=h; }

  XSetForeground(display, draw_pixmapGC, topColor);

  XDrawLine(display, gfxPixmap, draw_pixmapGC, x1, y2, x1, y1);
  XDrawLine(display, gfxPixmap, draw_pixmapGC, x1, y1, x2, y1);

  XSetForeground(display, draw_pixmapGC, bottomColor);

  XDrawLine(display, gfxPixmap, draw_pixmapGC, x2, y1+1, x2, y2);
  XDrawLine(display, gfxPixmap, draw_pixmapGC, x2, y2, x1+1, y2);  
}


/************************************************************************
 *									*
 * display Game Over and winner if one exists			 	*
 *  									*
 ************************************************************************/

void displayGameOver(Boolean won, short playerNr, short x1, short y1,
		     short x2, short y2)
{
  int x_1, y_1, x_2, y_2;
  
  if(won==True)
  {
    x_1=x1*FIELD_SIZE +BORDER_SIZE+1+IMAGE_SIZE/2;
    y_1=y1*FIELD_SIZE +BORDER_SIZE+1+IMAGE_SIZE/2; 
    x_2=x2*FIELD_SIZE +BORDER_SIZE+1+IMAGE_SIZE/2;
    y_2=y2*FIELD_SIZE +BORDER_SIZE+1+IMAGE_SIZE/2;
    
  
    XSetForeground(display, win_pixmapGC, XGoGame.Player[playerNr].Color);
    XDrawLine(display, gfxPixmap, win_pixmapGC, x_1, y_1, x_2, y_2);
   
    XCopyArea(display, gfxPixmap, XtWindow(gfxWidget), image_windowGC,
 	      0, 0, XGoGame.Width, XGoGame.Height, 0, 0);

    showWinMessage(won, playerNr);
    
    if(XGoGame.Player[playerNr].Type==PLAYER_COMPUTER && 
       XGoGame.Player[playerNr^1].Type==PLAYER_HUMAN)
      playSound(LOSE_SOUND);
    else if(XGoGame.Player[playerNr].Type==PLAYER_HUMAN && 
       XGoGame.Player[playerNr^1].Type==PLAYER_COMPUTER)
      playSound(WIN_SOUND);      
    else
      playSound(OVER_SOUND);
  }
  else
  {
    showWinMessage(won, playerNr);
    playSound(OVER_SOUND);
  }
}


/************************************************************************
 *									*
 * show a winner message					 	*
 *  									*
 ************************************************************************/

static void showWinMessage(Boolean won, short playerNr)
{
  static Widget winW=NULL;
  static Widget dialogShell=NULL;
  Arg           args[10];
  Cardinal      n;

  if(winW==NULL)
  {
    Widget	childW;
    XmString	cstring;
    
    cstring=XmStringCreateSimple("Game Over");
    
    n=0;
    XtSetArg(args[n], XmNautoUnmanage, False); 				n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); 	n++;
    XtSetArg(args[n], XmNdialogTitle, cstring);				n++;
    winW=XmCreateInformationDialog(applShell, "winInfo", args, n);
    XmStringFree(cstring);
    
    dialogShell=XtParent(winW);

    XtAddCallback(winW, XmNokCallback, popdownCB, (XtPointer)dialogShell);

    XtUnmanageChild(XmMessageBoxGetChild(winW,XmDIALOG_CANCEL_BUTTON));    
    XtUnmanageChild(XmMessageBoxGetChild(winW,XmDIALOG_HELP_BUTTON));

    childW=XmMessageBoxGetChild(winW, XmDIALOG_MESSAGE_LABEL);
    
    n=0;
    XtSetArg(args[n], XmNalignment, XmALIGNMENT_CENTER);	n++;
    XtSetValues(winW, args, n);

    setWinner(winW, won, playerNr);    

    setCloseCallback(dialogShell, (XtCallbackProc)popdownCB, (XtPointer)dialogShell);
     
    XtManageChild(winW);
  }
  else
  {
    setWinner(winW, won, playerNr);
    
    XtPopup(dialogShell, XtGrabExclusive);
  }
}


/************************************************************************
 *									*
 * set labelString for winner Message				 	*
 *  									*
 ************************************************************************/

static void setWinner(Widget widget, Boolean won, short playerNr)
{
  Widget 	childW;
  Arg	 	args[10];
  Cardinal 	n;
  XmString	cstring;
  
  if(won==FALSE)
    cstring=XmStringCreateSimple("The game ended draw !");
  else
  {
    XmString	cs1, cs2;
    
    cs1=XmStringCreateSimple("The winner is: ");
    cs2=XmStringCreateSimple(XGoGame.Player[playerNr].Name);
    cstring=XmStringConcat(cs1, cs2);
    
    XmStringFree(cs1);
    XmStringFree(cs2);
  }

  childW=XmMessageBoxGetChild(widget, XmDIALOG_MESSAGE_LABEL);
    
  n=0;
  XtSetArg(args[n], XmNlabelString, cstring);	n++;
  XtSetValues(childW, args, n);
  XmStringFree(cstring);
}


/************************************************************************
 *									*
 * free all pixmaps and images					 	*
 *  									*
 ************************************************************************/

void destroyAll()
{
  short i,j;
  
  XFreeGC(display, win_pixmapGC);
  XFreeGC(display, image_pixmapGC);
  XFreeGC(display, draw_pixmapGC);
  XFreeGC(display, image_windowGC);
  
  XFreePixmap(display, gfxPixmap);
  
  XDestroyImage(playerImage[0]);
  XDestroyImage(playerImage[1]);
  XDestroyImage(playerImage[2]);
  
  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      XDestroyImage(undoImage[i][j]);
}


/************************************************************************
 *									*
 * Expose callback procedure for the drawing area 			*
 *   this procedure restores the drawing area, when it has been 	*
 *   destroyed								*
 ************************************************************************/

void exposeCB(Widget widget, XtPointer client_data, XtPointer call_data)
{
  XmDrawingAreaCallbackStruct *DAinfo;
  
  DAinfo=(XmDrawingAreaCallbackStruct *)call_data;
 
  XCopyArea(display, gfxPixmap, XtWindow(widget), image_windowGC,
	    DAinfo->event->xexpose.x, DAinfo->event->xexpose.y,
	    DAinfo->event->xexpose.width, DAinfo->event->xexpose.height,
	    DAinfo->event->xexpose.x, DAinfo->event->xexpose.y);
}


/************************************************************************
 *									*
 * Input callback procedure for drawing area:			 	*
 *   this procedure recognizes moves made by human players 		*
 ************************************************************************/

void gfxinputCB(Widget widget, XtPointer client_data, XtPointer call_data)
{
  XmDrawingAreaCallbackStruct 	*DAInfo;
  int				x, y;
  
  DAInfo=(XmDrawingAreaCallbackStruct *)call_data;
  
  if(DAInfo->event->type==ButtonRelease && XGoGame.GameStatus!=GAME_FINISHED &&
     XGoGame.Player[XGoGame.PlayersTurn].Type==PLAYER_HUMAN)
  {
    if(DAInfo->event->xbutton.button==Button1 || 
       DAInfo->event->xbutton.button==Button3)
    {
      x=DAInfo->event->xbutton.x-BORDER_SIZE;
      y=DAInfo->event->xbutton.y-BORDER_SIZE;
      
      x/=FIELD_SIZE;
      y/=FIELD_SIZE;
      if(x>=0 && x<XGoGame.Columns && y>=0 && y<XGoGame.Rows)
      {
	if(DAInfo->event->xbutton.button==Button1)
	{  
	  if(getField(x,y)==FIELD_EMPTY)
	    makeMove(x,y);
	  else
	    playSound(ILLEGAL_SOUND);
        }
      }
    }
  }
}
