/* XGoBang: (c) 1995-2008 by Andreas S. Krebs */

#include "xmgobang.h"

/* local prototypes */

static void initWidgets(Widget);

/* global variables  */

XtAppContext	context;
Display 	*display;
Widget   	applShell;
  
GoPlayer	XGoPlayer[2];
GoGame		XGoGame;


/* local variables */

static XrmOptionDescRec options[]=
{
  { "-player1", ".playerOneType", XrmoptionSepArg, NULL },
  { "-player2", ".playerTwoType", XrmoptionSepArg, NULL },
  { "-name1", ".playerOneName", XrmoptionSepArg, NULL },
  { "-name2", ".playerTwoName", XrmoptionSepArg, NULL },
  { "-size", ".boardSize", XrmoptionSepArg, NULL }
};

static String fallback_resources[]=
{
  "XmGoBang.title: XmGoBang by Andreas S. Krebs",
  "XmGoBang*timeOne.labelString: 00:00:00",
  "XmGoBang*timeTwo.labelString: 00:00:00",
  "XmGoBang*toMove.labelString:	to move",
  NULL
};


/************************************************************************
 *									*
 * initialize Xt, get resources, create widgets and start XtAppMainLoop	*
 *									*
 ************************************************************************/

main(int argc, char **argv)
{
  Arg	   args[10];
  Cardinal n;

  applShell=XtAppInitialize(&context, APPL_CLASS, (XrmOptionDescRec *)options,
			    XtNumber(options), &argc, argv, 
			    fallback_resources, (Arg *)NULL, 0);

  display=XtDisplay(applShell);
  getApplResources(applShell);

  initWidgets(applShell);
  createXImages();
  allocBoard();

  setCloseCallback(applShell, (XtCallbackProc)exitQuestionCB, NULL);

  XtRealizeWidget(applShell);

  newGame();

  XtAppMainLoop(context);
}


/************************************************************************
 *									*
 * create main window widget and child widgets   			*
 *									*
 ************************************************************************/

static void initWidgets(Widget applShell)
{
  Widget mainW, menuW, formW, panedW;
  
  mainW=XtCreateManagedWidget("main", xmMainWindowWidgetClass, applShell, 
			      (Arg *)NULL, 0);

  menuW=initMenuWidget(mainW);
  formW=initFormWidget(mainW);

  XmMainWindowSetAreas(mainW, menuW, (Widget)NULL, (Widget)NULL, 
		       (Widget)NULL, formW);
}


/************************************************************************
 *									*
 * Callback procedure to exit the program:			 	*
 *  									*
 ************************************************************************/

void exitCB(Widget widget, XtPointer client_data, XtPointer call_data)
{
  playSound(GOODBYE_SOUND);
  
  freeBoard();
  destroyAll();
  destroyCursors();

#ifdef SOUND_SUPPORT  
  kill_soundchild();
#endif
  
  exit(GOOD);
}


/************************************************************************
 *									*
 * sets the callback to call, when user selects close from window menu	*
 *  									*
 ************************************************************************/

void setCloseCallback(Widget shell, XtCallbackProc cb, XtPointer client_data)
{
  Atom		wm_delete_window;
  Arg		args[10];
  Cardinal	n;
  
  wm_delete_window=XmInternAtom(display, "WM_DELETE_WINDOW", False);
  XmAddWMProtocolCallback(shell, wm_delete_window, cb, client_data);

  n=0;
  XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING);	n++;
  XtSetValues(shell, args, n); 
}











